@@ -23,6 +23,7 @@ import dns.resolver
 import dns.reversename
 import dns.message
 import socket
+import random
 from dns.zone import *
 from dns.dnssec import algorithm_to_text
 from .msf_print import *
@@ -38,7 +39,9 @@ class DnsHelper:
         self._proto = proto
         if ns_server:
             self._res = dns.resolver.Resolver(configure=False)
-            self._res.nameservers = [ns_server]
+            self._res.nameservers = ns_server
+            if len(ns_server) > 1:
+                self._res.rotate = True
         else:
             self._res = dns.resolver.Resolver(configure=True)
         # Set timing
@@ -76,10 +79,17 @@ class DnsHelper:
         return answers
 
     def query(self, q, where, timeout=None, port=53, af=None, source=None, source_port=0, one_rr_per_rrset=False):
-        if self._proto == "tcp":
-            return dns.query.tcp(q, where, timeout, port, af, source, source_port, one_rr_per_rrset)
+
+        if isinstance(where, list):
+            random.shuffle(where)
+            target_server = where[0]
         else:
-            return dns.query.udp(q, where, timeout, port, af, source, source_port, False, one_rr_per_rrset)
+            target_server = where
+
+        if self._proto == "tcp":
+            return dns.query.tcp(q, target_server, timeout, port, af, source, source_port, one_rr_per_rrset)
+        else:
+            return dns.query.udp(q, target_server, timeout, port, af, source, source_port, False, one_rr_per_rrset)
 
     def get_a(self, host_trg):
         """
@@ -207,33 +217,45 @@ class DnsHelper:
                 response = dns.query.tcp(querymsg, self._res.nameservers[0], self._res.timeout)
             else:
                 response = dns.query.udp(querymsg, self._res.nameservers[0], self._res.timeout)
-                
-            if len(response.answer) > 0:
-                answers = response.answer
-            elif len(response.authority) > 0:
+
+            if len(response.authority) > 0:
                 answers = response.authority
+            elif len(response.answer) > 0:
+                answers = response.answer
+            else:
+                return soa_records
+
             for rdata in answers:
-                # A zone only has one SOA record so we select the first.
-                name = rdata[0].mname.to_text()
-                ipv4_answers = self._res.query(name, 'A', tcp=tcp)
-                for ardata in ipv4_answers:
-                    if name.endswith('.'):
-                        soa_records.append(['SOA', name[:-1], ardata.address])
-                    else:
-                        soa_records.append(['SOA', name, ardata.address])
+                name = None
+                if isinstance(rdata[0], dns.rdtypes.ANY.SOA.SOA):
+                    name = rdata[0].mname.to_text()
+                else:
+                    continue
+
+                if name:
+                    ipv4_answers = self._res.query(name, 'A', tcp=tcp)
+                    for ardata in ipv4_answers:
+                        if name.endswith('.'):
+                            soa_records.append(['SOA', name[:-1], ardata.address])
+                        else:
+                            soa_records.append(['SOA', name, ardata.address])
         except (dns.resolver.NXDOMAIN, dns.exception.Timeout, dns.resolver.NoAnswer, socket.error, dns.query.BadResponse):
             print_error('Error while resolving SOA record.')
             return soa_records
 
         try:
             for rdata in answers:
-                name = rdata.mname.to_text()
-                ipv4_answers = self._res.query(name, 'AAAA', tcp=tcp)
-                for ardata in ipv4_answers:
-                    if name.endswith('.'):
-                        soa_records.append(['SOA', name[:-1], ardata.address])
-                    else:
-                        soa_records.append(['SOA', name, ardata.address])
+                name = None
+                if isinstance(rdata, dns.rdtypes.ANY.SOA.SOA):
+                    name = rdata.mname.to_text()
+
+                if name:
+                    ipv6_answers = self._res.query(name, 'AAAA', tcp=tcp)
+                    for ardata in ipv6_answers:
+                        if name.endswith('.'):
+                            soa_records.append(['SOA', name[:-1], ardata.address])
+                        else:
+                            soa_records.append(['SOA', name, ardata.address])
 
             return soa_records
         except:
